require "import"
import "android.widget.*"
import "android.view.*"
import "android.app.*"
import "android.provider.ContactsContract"
import "android.net.Uri"
import "android.Manifest"
import "android.content.pm.PackageManager"
import "android.content.Intent"
import "java.util.*"
import "android.content.Context"
import "android.content.Intent"
import "java.util.Locale"
import "android.speech.SpeechRecognizer"
import "android.speech.RecognitionListener"
import "android.speech.RecognizerIntent"
import "android.os.Build"
import "android.os.VibrationEffect"
import "android.os.Vibrator"
import "android.graphics.BitmapFactory"
import "java.net.URL"
import "java.io.InputStream"
import "android.os.SystemClock"
import "android.content.SharedPreferences"

local activity = activity
local contactList = {}
local contactsSpinner = nil
local selectedContactNumber = nil
local starredContacts = {}
local VERSION = "2.9"

function onCreate(savedInstanceState)
    activity.setTitle("Speed Dial " .. VERSION)
    local layout = LinearLayout(activity)
    layout.setOrientation(1)

    -- Request contacts permissions
    if not checkContactPermissions() then
        requestContactPermissions()
    else
        loadContacts(layout)
        loadManualDialer(layout)
        loadImage(layout)
        loadContactActions(layout)

        -- Start voice search automatically for no more than 5 seconds
        local startTime = SystemClock.elapsedRealtime()
        local maxDuration = 5000 -- 5 seconds
        local runnable = Runnable {
            run = function()
                local elapsedTime = SystemClock.elapsedRealtime() - startTime
                if elapsedTime <= maxDuration then
                    startVoiceSearch()
                end
            end
        }
        activity.runOnUiThread(runnable)

        -- Load starred contacts from SharedPreferences
        loadStarredContacts()
    end

    activity.setContentView(layout)
end

function loadContactActions(layout)
    -- Create the "Share Contacts as Text" button
    local shareContactsTextButton = Button(activity)
    shareContactsTextButton.setText("Share Contacts as Text")
    shareContactsTextButton.setOnClickListener{
        onClick = function(v)
            shareContactsAsText()
            vibrateDevice()
        end
    }
    layout.addView(shareContactsTextButton)

    -- Create the "Display Contact Number" button
    local displayContactNumberButton = Button(activity)
    displayContactNumberButton.setText("Display Contact Number")
    displayContactNumberButton.setOnClickListener{
        onClick = function(v)
            local selectedPosition = contactsSpinner.getSelectedItemPosition()
            selectedContactNumber = contactList[selectedPosition + 1].phoneNumber
            displayContactNumber()
            vibrateDevice()
        end
    }
    layout.addView(displayContactNumberButton)

    -- Create the "Star Contact" button
    local starContactButton = Button(activity)
    starContactButton.setText("Star Contact")
    starContactButton.setOnClickListener{
        onClick = function(v)
            local selectedPosition = contactsSpinner.getSelectedItemPosition()
            local contactId = contactList[selectedPosition + 1].id
            toggleStarContact(contactId)
            populateContacts(contactsSpinner)
            vibrateDevice()
        end
    }
    layout.addView(starContactButton)
end

function shareContactsAsText()
    -- Create a string with contact information
    local contactText = ""
    for _, contact in ipairs(contactList) do
        contactText = contactText .. contact.name .. " - " .. contact.phoneNumber .. "\n"
    end

    -- Share the contact information as text
    local intent = Intent(Intent.ACTION_SEND)
    intent.setType("text/plain")
    intent.putExtra(Intent.EXTRA_TEXT, contactText)
    activity.startActivity(Intent.createChooser(intent, "Share Contacts"))
    showToast("Sharing contacts as text")
end

function displayContactNumber()
    if selectedContactNumber then
        -- Display the selected contact number in a dialog
        local builder = AlertDialog.Builder(activity)
        builder.setTitle("Contact Number")
        builder.setMessage(selectedContactNumber)
        builder.setPositiveButton("OK", nil)
        builder.show()
        showToast("Displaying contact number")
    else
        showToast("No contact selected")
    end
end

function checkContactPermissions()
    return activity.checkSelfPermission(Manifest.permission.READ_CONTACTS) == PackageManager.PERMISSION_GRANTED
end

function requestContactPermissions()
    activity.requestPermissions({ Manifest.permission.READ_CONTACTS }, 1)
end

function loadContacts(layout)
    -- Create the contact selection dropdown
    contactsSpinner = Spinner(activity)
    layout.addView(contactsSpinner)

    -- Create the call button
    local callButton = Button(activity)
    callButton.setText("Call Contact")
    callButton.setOnClickListener{
        onClick = function(v)
            local selectedPosition = contactsSpinner.getSelectedItemPosition()
            selectedContactNumber = contactList[selectedPosition + 1].phoneNumber
            callContact(selectedContactNumber)
            vibrateDevice()
        end
    }
    layout.addView(callButton)

    -- Create the voice search button
    local voiceSearchButton = Button(activity)
    voiceSearchButton.setText("Voice Search")
    voiceSearchButton.setOnClickListener {
        onClick = function(v)
            startVoiceSearch()
            vibrateDevice()
        end
    }
    layout.addView(voiceSearchButton)

    -- Load contacts
    populateContacts(contactsSpinner)
end

function loadManualDialer(layout)
    -- Create the manual dial AutoCompleteTextView
    local manualDialEditText = AutoCompleteTextView(activity)
    manualDialEditText.setHint("Enter phone number")
    local adapter = ArrayAdapter(activity, android.R.layout.simple_dropdown_item_1line, {})
    manualDialEditText.setAdapter(adapter)
    layout.addView(manualDialEditText)

    -- Create the manual dial button
    local manualDialButton = Button(activity)
    manualDialButton.setText("Call Manual")
    manualDialButton.setOnClickListener{
        onClick = function(v)
            local phoneNumber = manualDialEditText.getText().toString()
            callContact(phoneNumber)
            vibrateDevice()
        end
    }
    layout.addView(manualDialButton)

    -- Populate the contact suggestions
    populateContactSuggestions(adapter)
end

function loadImage(layout)
    -- Create the image view
    local imageView = ImageView(activity)
    local imageUrl = "https://image.pollinations.ai/prompt/Square,%20Rainbow%20colors%20wait%20in%20the%20air?width=300&height=300&nologo=true"
    local bitmap = loadBitmapFromUrl(imageUrl)
    imageView.setImageBitmap(bitmap)
    imageView.setContentDescription("Rainbow colors wait in the air")
    layout.addView(imageView)
end

function loadBitmapFromUrl(imageUrl)
    local url = URL(imageUrl)
    local connection = url.openConnection()
    local inputStream = connection.getInputStream()
    return BitmapFactory.decodeStream(inputStream)
end

function populateContacts(contactsSpinner)
    local cursor = activity.getContentResolver().query(ContactsContract.Contacts.CONTENT_URI, nil, nil, nil, nil)
    local adapter = ArrayAdapter(activity, android.R.layout.simple_spinner_item, {})
    contactsSpinner.setAdapter(adapter)

    contactList = {}
    while cursor.moveToNext() do
        local id = cursor.getString(cursor.getColumnIndex(ContactsContract.Contacts._ID))
        local name = cursor.getString(cursor.getColumnIndex(ContactsContract.Contacts.DISPLAY_NAME))
        local phoneNumber = getPhoneNumber(id)

        if phoneNumber then
            local isStarred = isContactStarred(id)
            if isStarred then
                adapter.insert(name, 0)
            else
                adapter.add(name)
            end
            table.insert(contactList, {id = id, name = name, phoneNumber = phoneNumber, isStarred = isStarred})
        end
    end
    cursor.close()
    adapter.notifyDataSetChanged()
end

function populateContactSuggestions(adapter)
    local cursor = activity.getContentResolver().query(ContactsContract.CommonDataKinds.Phone.CONTENT_URI, nil, nil, nil, nil)
    while cursor.moveToNext() do
        local phoneNumber = cursor.getString(cursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.NUMBER))
        adapter.add(phoneNumber)
    end
    cursor.close()
    adapter.notifyDataSetChanged()
end

function getPhoneNumber(contactId)
    local cursor = activity.getContentResolver().query(ContactsContract.CommonDataKinds.Phone.CONTENT_URI, nil, ContactsContract.CommonDataKinds.Phone.CONTACT_ID .. " = ?", {contactId}, nil)
    if cursor.moveToFirst() then
        local phoneNumber = cursor.getString(cursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.NUMBER))
        cursor.close()
        return phoneNumber
    end
    cursor.close()
    return nil
end

function callContact(phoneNumber)
    local intent = Intent(Intent.ACTION_CALL)
    intent.setData(Uri.parse("tel:" .. phoneNumber))
    activity.startActivity(intent)
end

function startVoiceSearch()
    local speechRecognizer = SpeechRecognizer.createSpeechRecognizer(activity)

    local recognitionListener = RecognitionListener {
        onResults = function(results)
            local resultArray = results.getParcelableArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
            if resultArray ~= nil and resultArray.size() > 0 then
                local searchQuery = resultArray.get(0)
                searchContacts(searchQuery)
                showToast("Voice search successful: " .. searchQuery)
            else
                showToast("Voice search failed")
            end
            destroySpeechRecognizer(speechRecognizer)
            vibrateDevice()
        end,
        onError = function()
            destroySpeechRecognizer(speechRecognizer)
            showToast("Voice search error")
            vibrateDevice()
            return false
        end
    }

    local recognizerIntent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, Locale.getDefault().getLanguage())
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_CALLING_PACKAGE, activity.getPackageName())

    speechRecognizer.startListening(recognizerIntent)
    speechRecognizer.setRecognitionListener(recognitionListener)
end

function destroySpeechRecognizer(rec)
    if rec then
        rec.destroy()
        rec = nil
    end
    return true
end

function searchContacts(searchQuery)
    if contactsSpinner ~= nil then
        local cursor = activity.getContentResolver().query(ContactsContract.Contacts.CONTENT_URI, nil, ContactsContract.Contacts.DISPLAY_NAME .. " LIKE ?", {"%" .. searchQuery .. "%"}, nil)
        local adapter = ArrayAdapter(activity, android.R.layout.simple_spinner_item, {})
        contactsSpinner.setAdapter(adapter)

        contactList = {}
        while cursor.moveToNext() do
            local id = cursor.getString(cursor.getColumnIndex(ContactsContract.Contacts._ID))
            local name = cursor.getString(cursor.getColumnIndex(ContactsContract.Contacts.DISPLAY_NAME))
            local phoneNumber = getPhoneNumber(id)

            if phoneNumber then
                local isStarred = isContactStarred(id)
                if isStarred then
                    adapter.insert(name, 0)
                else
                    adapter.add(name)
                end
                table.insert(contactList, {id = id, name = name, phoneNumber = phoneNumber, isStarred = isStarred})
            end
        end
        cursor.close()
        adapter.notifyDataSetChanged()
    else
        -- Handle the case where contactsSpinner is nil
        -- You can display an error message or perform some other action
    end
end

function vibrateDevice()
    local vibrator = activity.getSystemService(Context.VIBRATOR_SERVICE)
    if Build.VERSION.SDK_INT >= Build.VERSION_CODES.O then
        vibrator.vibrate(VibrationEffect.createOneShot(200, VibrationEffect.DEFAULT_AMPLITUDE))
    else
        vibrator.vibrate(200)
    end
end

function showToast(message)
    local toast = Toast.makeText(activity, message, Toast.LENGTH_SHORT)
    toast.show()
end

function isContactStarred(contactId)
    return starredContacts[contactId] ~= nil
end

function toggleStarContact(contactId)
    if isContactStarred(contactId) then
        starredContacts[contactId] = nil
    else
        starredContacts[contactId] = true
    end
    saveStarredContacts()
end

function loadStarredContacts()
    local sharedPreferences = activity.getSharedPreferences("speed_dial_app", Context.MODE_PRIVATE)
    local starredContactsString = sharedPreferences.getString("starred_contacts", "")
    if starredContactsString ~= "" then
        local starredContactIds = split(starredContactsString, ",")
        for _, contactId in ipairs(starredContactIds) do
            starredContacts[contactId] = true
        end
    end
end

function saveStarredContacts()
    local sharedPreferences = activity.getSharedPreferences("speed_dial_app", Context.MODE_PRIVATE)
    local editor = sharedPreferences.edit()
    local starredContactIds = {}
    for contactId, _ in pairs(starredContacts) do
        table.insert(starredContactIds, contactId)
    end
    local starredContactsString = table.concat(starredContactIds, ",")
    editor.putString("starred_contacts", starredContactsString)
    editor.apply()
end

function split(str, sep)
    local t = {}
    for match in (str .. sep):gmatch("(.-)" .. sep) do
        table.insert(t, match)
    end
    return t
end